	SUBROUTINE  get_vpt(jet_vpt,npvert,jet_novpt,ptnov)
C======================================================================
C
C===Description:
C
C	Returns seed primary vertex, based on highest-Q z-VTVZ vertex
C       (cdf-2084) + SVX beam line, to be used for PVF combined fit
C       (cdf-2055) or as initial value for PVF standard fit (cdf-1789).
C
C   SVX BEAM LINE IN B0 DATA: for B0 data the SVX beam line is picked up
C	from c$inc:svxbpo.inc if explicitily required (intercepts, slopes 
C       AND the beam spot size available: cdf-1924) or from the VTVZ itself
C	by default (only intercepts and slopes). 
C	All runs should have VTVZ filled with an SVX+CTC fit of the SVX 
C       beam line. If not (may be 6.0X data) select full SVX BPO.
C       All run are also supposed to have the offline computed SVX beam 
C       line in c$inc:svxbpo.inc (after the proper call to c$svx:svgebp.cdf; 
C       if vxprim module is used, c$svx:svgebp.cdf is called in the vxprim 
C       run init routine).
C
C   BEAM LINE IN MONTECARLO DATA: for MC data, the beam line is picked
C       up from SIMH bank. The vxprim module does not call svgebp.cdf
C       for MC data (that is, if SIMH is belocated succesfully).
C
C
C	** npvert = 0 if no VTVZ z-info is available
C	** blinfo = .false. if no beam line is available 
C
C===Input Arguments:
C
C               None
C
C===Output Arguments:
C
C	INTEGER npvert		number of candidate seed PVs.
C			        npvert = MIN(nmax,nvtvz), where
C				nmax = 5, nvtvz = number of PVs listed
C				in VTVZ. Coordinates and cov. matrices
C				of these npvert candidates are stored
C				in xseed and cvseed, but since the first
C				such candidate in VTVZ is the 'best',
C				(see c$doc:daisvtvz.mem), that one is
C				used as seed for the combined fit (and
C				returned as output argument) by the
C				calling routine c$vtx:vxgtpr.cdf. The
C				remaining npvert-1 candidates are saved
C				in xseed/cvseed for comparison, monitoring
C				and debugging purposes or future use.
C
C				If VTVZ  z-info is NOT available
C				(the SVX beam line then is NOT usable)
C				npvert = 0, xseed, cvseed=defaults.
C
C	LOGICAL blinfo		flags SVX beam line availability.
C				blinfo = .false. if the SVX beam line
C				is not available
C
C				NOTICE: if npvert=0 or blinfo=.false.
C				the calling routine, vxgtpr, will do the
C				standard PVF fit (cdf-1789), regardless
C				of users' request of a combined PVF fit.
C
C       REAL xseed(nmax,3) 	seed PV coordinates for combined fit
C	REAL cvseed(nmax,6) 	seed PV covariance matrices
C	REAL Qint(nmax)		seed PV's track Pt scalar sum; highest
C				Qint PV is to be used by VXGTPR/VXPRIM
C
C===Preconditions necessary before call:
C
C    For B0 data call c$svx:svgebp.cdf in your run init routine to have
C		c$inc:svxbpo.inc filled and used as SVX beam line.
C
C===Subroutines called:
C
C               None
C
C===Functions called:
C
C               None
C
C===Written for calling by:
C
C               PV-finder routine c$vtx:vxgtpr.cdf
C
C===Author:
C
C	S. Dell'Agnello
C	INFN and University of Pisa
C
C===Revision History:
C
C	Jul-1991  Original creation (cdf-1789)
C	Apr-1993  Extensive changes to implement combined fit (cdf-2055)
C	May-1993  Handling multiple PV's (cdf-2084)
C	Jun-1993  Getting 'beam line' from SIMH for MC data (cdf-2084)
C       Aug-1993  Modified for production 6.15    (F. Tartarelli)
C
C======================================================================
C
$$Implicit
C
C===Global declarations:
C
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:SIMH.INC'
$$INCLUDE 'C$INC:OBS203.INC'
$$INCLUDE 'C$INC:DSEVTA.INC'
$$INCLUDE 'C$INC:SVXBPO.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:CTCALB.INC'
$$INCLUDE 'C$INC:VXPVIN.INC'
$$INCLUDE 'C$INC:TRKSCO.INC'
$$INCLUDE 'C$INC:TRFITC.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:DSFITP.INC'
$$INCLUDE 'C$INC:JETINFO.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
C
C===Argument Declarations:
C
	INTEGER npvert	       ! number of seed PV candidates
	INTEGER nmax	       ! max number of seed PV candidates
	PARAMETER (nmax = 20)
	REAL xseed(nmax,3)     ! coordinates of seed PV candidates
	REAL cvseed(nmax,6)    ! covariance matrices of seed PV candidates
	REAL Qint(nmax)	       ! track scalar Pt sum of seed PV's
	LOGICAL blinfo	       ! flags SVX beam line availability
	REAL dz0cut	       ! cut on track-PV zmatch
C
C===Local Declarations:
C
	REAL x0,y0,dxdz,dydz   ! CTC beam pos. & slopes at z=0 from VTVZ
	INTEGER indevta,datevta! EVTA bank and data indices
	INTEGER indvtvz,datvtvz! VTVZ bank and data indices
	INTEGER nword,nwdv,nvtvz! VTVZ stuff
	INTEGER i	       ! dummy
	INTEGER status	       ! YBOS return code
	LOGICAL vtvzpv	       ! flags VTVZ z info availability
	INTEGER vclass(nmax)      ! seed PV vertex class (VTVZ class)
        INTEGER nlimit,ivsno
C
C-------Qtrkct.cdf declarations
C
	INTEGER itrks,charge,hitlay,numout,ierr,isl,naxl,nstl,ntrks
	REAL px,py,pz,z0,impact,dz0,pt
	LOGICAL flag3d,vtc,svx
C
C-------SIMH declarations
C
	INTEGER simind,simdat,ierrs,ierro,junk1,junk2
	LOGICAL mcdata
C
C===External Declarations:
C
	INTEGER blocat,qtrkun
        real cdfphi,prapid
C
C   jwb declarations
        real jet_vpt(nmax,20),jet_novpt(20),ptnov
        real xcal(3),phitr,dphi,etatr,deta
        integer nvert,ij
        logical nomatch
C===Data Statements:
C
C===Error messages:
C
C===Start of Code:
C
C
       
        

C-------Defaults if no VTVZ z-info or no SVX beam line info
C
	npvert = 0
	blinfo = .false.
	CALL vzero(xseed,nmax*3)
      	CALL vzero(Qint,nmax)	    ! init Qint for all PV's
	DO i=1,nmax
	  cvseed(i,1) = 1.E-2       ! x rms = 1 mm
	  cvseed(i,2) = 0.
	  cvseed(i,3) = 0.
	  cvseed(i,4) = 1.E-2       ! y rms = 1 mm
	  cvseed(i,5) = 0.
	  cvseed(i,6) = 30.         ! z rms = 30 cm
	ENDDO
C
C    set simulation flag
C                                               
	mcdata = .false.
        status = blocat(iw,'SIMH',-1,simind,simdat)
        IF (status.EQ.yesucc) THEN
	  mcdata = .true.
	ELSE
	  status = blocat(iw,'OBSP',-1,junk1,junk2)
	  IF(status.eq.yesucc) mcdata = .true.
	ENDIF
C
C-------Use VTVZ bank if any
C         
	status = blocat(iw,'VTVZ',-1,indvtvz,datvtvz)
	IF (status.NE.yesucc) GOTO 999   ! return with error message
C
C   check VTVZ content
C
	nword = iw(datvtvz)
	nvtvz = iw(datvtvz+1)
	nwdv  = iw(datvtvz+2)
	ivsno = iw(datvtvz + 3)
C
C    VTVZ is empty: return with error status 
C
        IF (nvtvz.LE.0) GOTO 999
C                                                  
C
C    It's an unknown VTVZ version and there is no way to recover
C   
        IF((ivsno.LT.2) .AND. .NOT.(svfull.AND.svfobm) .AND.
     +                        .NOT. mcdata) GOTO 999
C
C---------- z vertex info from VTVZ 
C
	nlimit = MIN(nvtvz,nmax)	! unpack at most nlimit PV's
C
        CALL vtgzzt(nlimit,npvert,vclass,xseed(1,3),xseed(1,1),
     +                     xseed(1,2))
C 
C       for all vtvz banks calculate sum of scalar momentum
C
        dz0cut = zmatch
        nvert = 0
        do i = 1,nmax
        do ij = 1,20
          jet_vpt(i,ij) = 0.0
        enddo
        enddo
        DO i=1,npvert
          cvseed(i,6) = rw(datvtvz+nwdv*(i-1) + 7)**2   ! z rms
          IF (cvseed(i,6).LE.0.) cvseed(i,6) = 0.25**2  ! z rms=2.5 mm
            nvert = nvert + 1
            ntrks = qtrkun(0,0)           ! get total number of tracks
            DO itrks=1,ntrks
              status = qtrkun(itrks,1)    ! full QTRK unpack track by track
              IF (status .EQ. 0) THEN
                flag3d = iand(trkspc,tpmfc).EQ.TP02FC
                IF (flag3d) THEN          ! if track is 3D
                    z0 = trksfp(3)              ! retrieve z0 of track
                    dz0 = z0-xseed(i,3)
                    IF (ABS(dz0).LT.dz0cut) THEN !track-PV z distance
                      Pt =  trksp4(5)           ! retrieve track pt
                        Qint(i) = Qint(i) + Pt       ! track scalar Pt sum
C track matching for jets
               do ij = 1,min(20,iclus)
                 phitr = cdfphi(atan2(trksp4(2),trksp4(1)))
                 dphi =phitr-ijet_phi(ij)
                 if(abs(dphi).gt.PI) dphi = TWOPI-abs(dphi)
                 call emxrt(trksfp(cufp),trksfp(d0fp),phitr,
     &                      trksfp(ctfp),xseed(i,3),0,xcal)
                 etatr = prapid(xcal)
                 deta = etatr - ijet_deta(ij)
                 if(sqrt(dphi**2+deta**2).lt.0.4) then
                 jet_vpt(nvert,ij) = jet_vpt(nvert,ij) + Pt
                 Endif              ! cone cut
               enddo
                    ENDIF
                ENDIF
              ENDIF
            ENDDO
        ENDDO ! end loop on npvert VTVZ PV's

        do i = 1,20
          jet_novpt(i) = 0.0
        enddo
        ptnov = 0.0

        ntrks = qtrkun(0,0)           ! get total number of tracks
        DO itrks=1,ntrks
              nomatch = .true.
              status = qtrkun(itrks,1)    ! full QTRK unpack track by track
              IF (status .EQ. 0) THEN
                flag3d = iand(trkspc,tpmfc).EQ.TP02FC
                IF (flag3d) THEN          ! if track is 3D
                    z0 = trksfp(3)              ! retrieve z0 of track
        do i = 1,npvert 
           dz0 = z0-xseed(i,3)
           IF (ABS(dz0).LT.dz0cut) nomatch = .false. !track-PV z distance
        enddo
          Pt =  trksp4(5)           ! retrieve track pt
          IF (pt.gt.0.0.and.nomatch) THEN     ! protection
               ptnov = ptnov + Pt       ! track scalar Pt sum
C track matching for jets
               do ij = 1,min(20,iclus)
                 phitr = cdfphi(atan2(trksp4(2),trksp4(1)))
                 dphi =phitr-ijet_phi(ij)
                 if(abs(dphi).gt.PI) dphi = TWOPI-abs(dphi)
                 call emxrt(trksfp(cufp),trksfp(d0fp),phitr,
     &                      trksfp(ctfp),xseed(i,3),0,xcal)
                 etatr = prapid(xcal)
                 deta = etatr - ijet_deta(ij)
                 if(sqrt(dphi**2+deta**2).lt.0.4) then
                 jet_novpt(ij) = jet_novpt(ij) + Pt
                 Endif              ! cone cut
               enddo !iclus
          ENDIF !nomatch
       endif !status 
       endif !flag3d
       enddo !ntrks

C
C-----------xy info
C
C    	MC data: get beam line from SIMH bank (slopes = 0.)
C
	IF (mcdata) THEN
	  blinfo = .true.
	  CALL UZERO(PINSIM,1,NPINSM)
	  CALL SMHBTC(IERRS)
	  IF(IERRS.EQ.0)THEN	! Got SIMH
	    x0 = pinsim(shxave)
	    y0 = pinsim(shyave)
	    dxdz = pinsim(shxslp)
	    dydz = pinsim(shyslp)
	    DO i=1,npvert
	      xseed(i,1) = x0 + dxdz*xseed(i,3)
	      xseed(i,2) = y0 + dydz*xseed(i,3)
	      cvseed(i,1) = pinsim(shxsig)**2
	      cvseed(i,4) = pinsim(shysig)**2
	    ENDDO
	  ELSE		! No SIMH so use OBSV
	    CALL OBSBTC(IERRO)
	    IF(IERRO.NE.0)THEN
	      CALL ERLOGR('VXPEST',ERSEVR,0,0,'No SIMH/OBS')
	      RETURN
	    ENDIF
	    DO i=1,npvert
	      xseed(i,1) = pvobs(1,1)
	      xseed(i,2) = pvobs(2,1)
	      cvseed(i,1) = 0.0035**2
	      cvseed(i,4) = 0.0035**2
	    ENDDO
	  ENDIF
	ELSE
C
C   B0 data: get beam line from VTVZ bank (slopes.NE.0.)
C
   	  IF (ivsno.GE.3) THEN  ! SVX+CTC fit available
      	    blinfo = .true.	       
	    CALL ctvtvz            
	    x0 = xbeam
	    dxdz = bmxlin
	    y0 = ybeam
	    dydz = bmylin
	    DO i=1,npvert
	      xseed(i,1) = x0 + xseed(i,3)*dxdz	! x coordinate
	      xseed(i,2) = y0 + xseed(i,3)*dydz	! y coordinate
	      cvseed(i,1) = 0.0035**2 ! x rms = 35 microns
	      cvseed(i,4) = 0.0035**2 ! y rms = 35 microns
      	    ENDDO   
	  ENDIF     
	ENDIF
C
C-------Here if z info is available: try xy info from c$inc:SVXBPO.INC
C	IF required, IF information is available, IF data run.
C		  
	IF (svfull.AND.svfobm.AND..NOT.mcdata) THEN
      	  blinfo = .true.
      	  DO i=1,npvert
	    xseed(i,1) = xsvbea+svslox*xseed(i,3)
	    xseed(i,2) = ysvbea+svsloy*xseed(i,3)
	    cvseed(i,1) = svsigx**2
	    cvseed(i,4) = svsigy**2
	  ENDDO
	ENDIF
C
C===Return to Caller:
C
C    normal termination
C	 
	RETURN
C                                                                 
C    return with error message
C
999     CALL erlogr('get_vpt',ereror,0,0,'VTVZ missing or not valid')
	RETURN
C                                                           
	END
