C***********************************************************************
	INTEGER FUNCTION QTRKBM(CHI2,CHARGE,PX,PY,PZ,ZVERT,IMPACT)
C Description
C	Do beam constraint on a QTRK track
C Input
C	CHI2: limit on individual chi-square for initial Z0 and D0.
C	    If either Z0 or D0 fail the cut, no constraint is done.
C	    A value of order 10 (or <0, see below) should be fine.
C	    If the chi-square test fails, the output variables are filled
C	    with un-constrained values.
C	TRKSCO.INC and QTRKCO.INC filled by QTRKUN (or TRKSUN)
C Output
C	CHARGE: particle charge.  Only the sign is relevant.
C	PX, PY, PZ: track 3-momentum
C	IMPACT: impact parameter
C	ZVERT: vertex position
C	QTRKBM: error code
C	     0: successful constraint
C		or CHI2<0 and track may be constrained
C	    <0: Track cannot be constrained
C		-2: Track is already vertex constrained
C		-1: insufficient information (e.g., bad covariance)
C	    >0: if vertex fails the chi-square cut.
C NOTE:
C	For CHI2<0, the covariance of TRKSCO.INC is filled and the
C	unconstrained parameters converted momenta.  No error code
C	is returned.
C Author
C	Aseet Mukherjee, FNAL
C.......................................................................
$$Implicit
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:DSFITP.INC'
$$INCLUDE 'C$INC:TRFITC.INC'
$$INCLUDE 'C$INC:TRKSCO.INC'
$$INCLUDE 'C$INC:QTRKCO.INC'
$$INCLUDE 'C$INC:CTCALB.INC'
$$INCLUDE 'C$INC:VTCAUX.INC'

$$INCLUDE 'C$INC:MUOINFO.INC'
$$INCLUDE 'C$INC:MIOINFO.INC'
$$INCLUDE 'stntuple$source:my_bc.inc'
C Arguments
	REAL CHI2
	INTEGER CHARGE
	REAL PX,PY,PZ,ZVERT,IMPACT
C External
	INTEGER QTRKCV
C Track parameters
	REAL CUFIT,P0FIT,D0FIT,Z0FIT,CTFIT
	EQUIVALENCE (CUFIT,TRKSFP(CUFP))
	EQUIVALENCE (P0FIT,TRKSFP(P0FP))
	EQUIVALENCE (D0FIT,TRKSFP(D0FP))
	EQUIVALENCE (CTFIT,TRKSFP(CTFP))
	EQUIVALENCE (Z0FIT,TRKSFP(Z0FP))
C Constrained values
	REAL Z0FIX,D0FIX
	REAL Z0DEL,D0DEL
C Weight/covariance matrices
	REAL VMAT(5,5),WMAT(5,5)
C Convert to output variables
	REAL PT
C Scratch
	REAL TMP1,TMP2,SPACE(5)
	INTEGER IMAT,JMAT,KMAT,STATUS

C.......................................................................
	IF (IAND(TRKSPC,VFITFC).NE.0) THEN
	    QTRKBM = -2
	    GOTO 500
	ENDIF
    	IF (IAND(TRKSFC,CTFLFC).EQ.0) GOTO 300
	QTRKBM = -1
	STATUS = QTRKCV()
	IF (STATUS.NE.0) GO TO 500
	CALL CTVTVZ
	IF (IVTVZ.LE.0) GO TO 500
	IF (CHI2.LE.0) THEN
	    QTRKBM = 0
	    GO TO 500
	END IF
C Choose Z vertex
	QTRKBM = 1
	IF (TRKSCV(DZ02FP-FPEMFP+1).GT.0) THEN
	    JMAT = IVTVZ + 6
	    Z0FIX = RW(JMAT)
	    Z0DEL = Z0FIX - Z0FIT
	    DO IMAT = 2,IW(IVTVZ+1)
		JMAT = JMAT + LVTVZ
		TMP2 = RW(JMAT) - Z0FIT
		IF (ABS(TMP2).LT.ABS(Z0DEL)) THEN
		    Z0FIX = RW(JMAT)
		    Z0DEL = TMP2
		END IF
	    END DO
	    IF (Z0DEL**2.GE.CHI2*TRKSCV(DZ02FP-FPEMFP+1)) GO TO 500
	ELSEIF (TRKSCV(DD02FP-FPEMFP+1).GT.0) THEN
	    Z0FIX = Z0FIT
	    Z0DEL = 0
	ELSE
	    QTRKBM = -1
	    GOTO 500
	END IF
C Choose impact parameter
	D0FIX = COS(P0FIT) * (YBEAM+BMYLIN*Z0FIX) -
     &	  SIN(P0FIT) * (XBEAM+BMXLIN*Z0FIX)
	D0DEL = D0FIX - D0FIT
	IF (D0DEL**2.GE.CHI2*TRKSCV(DD02FP-FPEMFP+1)) GO TO 500
C Load weight matrix
	KMAT = 1
	CALL VZERO(WMAT,5*5)
	DO IMAT = 1,5
	    IF (QTRKWT(KMAT).GT.0) THEN
		DO JMAT = IMAT,5
		    WMAT(JMAT,IMAT) = QTRKWT(KMAT)
		    WMAT(IMAT,JMAT) = QTRKWT(KMAT)
		    KMAT = KMAT + 1
		ENDDO
	    ELSE
		WMAT(IMAT,IMAT) = -1
		KMAT = KMAT + (6-IMAT)
	    ENDIF
	ENDDO
C Remove Z0 and D0, then invert to covariance
	CALL UCOPY(WMAT,VMAT,5*5)
	DO IMAT = 1,5
	    VMAT(IMAT,D0FP) = 0
	    VMAT(D0FP,IMAT) = 0
	    VMAT(IMAT,Z0FP) = 0
	    VMAT(Z0FP,IMAT) = 0
	ENDDO
	VMAT(D0FP,D0FP) = -1
	VMAT(Z0FP,Z0FP) = -1
	CALL RINV(5,VMAT,5,SPACE,STATUS)
	IF (STATUS.NE.0) THEN
	    QTRKBM = -1
	    RETURN
	ENDIF

C Change in parameters
	DO IMAT = 1,5
	    SPACE(IMAT) = WMAT(IMAT,D0FP)*D0DEL + WMAT(IMAT,Z0FP)*Z0DEL
	ENDDO
	DO IMAT = 1,5
	    IF (VMAT(IMAT,IMAT).GT.0) THEN
		TMP1 = 0
		DO JMAT = 1,5
		    TMP1 = TMP1 + SPACE(JMAT)*VMAT(JMAT,IMAT)
		END DO
		TRKSFP(IMAT) = TRKSFP(IMAT) - TMP1
	    ENDIF
	ENDDO
	Z0FIT = Z0FIX
	D0FIT = D0FIX
	QTRKBM = 0
	TRKSPC = TRKSPC + VF02FC
	GOTO 500

C VTX
300	QTRKBM = 0
	IF (CHI2.LE.0) GOTO 500
C Find Z vertex
	JMAT = IVTVZ + 6
	Z0FIX = RW(JMAT)
	Z0DEL = Z0FIX - Z0FIT
	DO IMAT = 2,IW(IVTVZ+1)
	    JMAT = JMAT + LVTVZ
	    TMP2 = RW(JMAT) - Z0FIT
	    IF (ABS(TMP2).LT.ABS(Z0DEL)) THEN
		Z0FIX = RW(JMAT)
		Z0DEL = TMP2
	    END IF
	END DO
	IF (VTXETA.LT.9999) THEN
	    CTFIT = SINH(VTXETA)
	    Z0FIT = Z0FIX
	ELSEIF (ABS(Z0DEL).LT.0.1) THEN
	    Z0FIT = Z0FIX
	ELSE
	    QTRKBM = 1
	    GOTO 500
	ENDIF
	TRKSPC = TRKSPC + VF02FC

C Put back into arguments
500	IF (ABS(CUFIT).LT.1.E-20) THEN
	    PT = 999.
	    CHARGE = 0
	ELSE
	    PT = 2.99796E-4/2 * BMAGNT / ABS(CUFIT)
	    CHARGE = SIGN(1.,CUFIT)
	END IF
	PX = PT * COS(P0FIT)
	PY = PT * SIN(P0FIT)
	PZ = PT * CTFIT
	ZVERT = Z0FIT
	IMPACT = D0FIT
C Also update TRKSCO
	TRKSP4(1) = PX
	TRKSP4(2) = PY
	TRKSP4(3) = PZ
	TRKSP4(4) = SQRT(PT**2+PZ**2)
	TRKSP4(5) = PT
	TRKSP4(6) = LOG(SQRT(1+CTFIT**2)+CTFIT)
         z0chi2 = -999.
         d0chi2 = -999.
         if(TRKSCV(DZ02FP-FPEMFP+1).gt.0.)
     *           z0chi2 = Z0DEL**2/TRKSCV(DZ02FP-FPEMFP+1)
         if(TRKSCV(DD02FP-FPEMFP+1).gt.0.)
     *           d0chi2 = D0DEL**2/TRKSCV(DD02FP-FPEMFP+1)
	END
