      INTEGER FUNCTION FIDELE(ELESBN, LUNIT, LFIDUC, FAILED)
C
C===Description:
C
C  This function determines whether the electron candidate specified
C  in the ELES bank with bank number ELESBN falls within the
C  fiducial region of the electromagnetic calorimeters.
C
C  The portions of the CEM that are not allowed are:
C
C   i)  local wedge x > +/- 21.0 cm from tower center
C  ii)  |Z| of cluster position < 9 cm (avoid the 90 degree crack)
C iii)  Tower 9
C  iv)  Tower 7 of chimney module
C
C  Note that there is no implicit requirement that there be a 3D
C  track associated with the candidate as i) and ii) are based on
C  the CES position information.
C
C  The portions of the PEM that are not allowed are:
C
C   i)  outer 2 eta annuli
C  ii)  inner 2 eta annuli
C iii)  5 degree border of each quadrant (1 phi pad)
C  iv)  16 dead channels
C
C  The portions of the FEM that are not allowed are:
C
C   i)  inner 5 eta annuli
C  ii)  5 degree border of each quadrant (1 phi pad) 
C
C===Preconditions necessary before call:
C
C  ELES bank present.
C
C===Input Arguments:
C
C  ELESBN (Integer*4)           ELES/CALS bank number of cluster.
C  LUNIT  (Integer*4)           Debug dump written to unit LUNIT if
C                               LUNIT>0
C
C===Output Arguments:
C
C  LFIDUC (Logical)             True if electron is in fiducial region.
C  FAILED (Integer*4)           Bit pattern of cuts that fail.
C                                 0     CEM failed phi crack cut
C                                 1     CEM failed 90 crack cut
C                                 2     CEM failed tower 9 cut
C                                 3     CEM No 3D track
C				  4     CEM Failed tower 7 chimney
C				  5	PEM outer eta annuli
C				  6     PEM inner eta annuli
C				  7     PEM phi cut
C				  8	PEM dead channel
C				  9	FEM inner eta annuli
C				 10	FEM phi cut
C
C===Function Return Value:
C
C  ERSUCC       Normal execution
C  ERWARN       ELES bank not found or illegal region (< 0 or > 2)
C
C===Author:
C
C       Pekka K. Sinervo
C       University of Pennsylvania and CDF
C       24-APR-1989
C	Modified by Marshall Miller to include PEM and FEM
C	2-JUN-1989
C	Modified by Marshall Miller to include tower type 5 in PEM 
C	and omit bad run cuts.
C	31-JAN-1990
C	Modified by Paul Derwent to make wedge cuts using the CES positions
C	and to remove the 3D track requirement.
C	18-Apr-1990
C
C===Implicit None Declaration:
C
$$IMPLICIT
C
C===Global Declarations:
C
$$Include 'YBOS$LIBRARY:ERRCOD.INC'
$$Include 'Error_Logger$Library:ERLEVL.INC'
$$Include 'C$INC:BCS   .INC'
$$Include 'C$INC:JOBSTA.INC'
$$Include 'C$INC:DSELES.INC'
$$Include 'C$INC:PIVALU.INC'
$$Include 'C$INC:TREXPO.INC'
$$Include 'C$INC:DSTOWE.INC'
C
C===Local Declarations:
C
        INTEGER         ELESBN, LUNIT
        LOGICAL         LFIDUC
        INTEGER         FAILED
	INTEGER		DEDNUM
	PARAMETER       (DEDNUM = 16)	 ! Number of dead PEM channels
	INTEGER	        DEDARR(DEDNUM,2)
        INTEGER         STATUS, IREG, I
        REAL            PHITRK, TEMP
        REAL            WEDGEX
        INTEGER         SEDETA, SEDPHI
	INTEGER		IPHI
        LOGICAL         INPHCR, IN90CR, TOWER9, NOTCEM, NOT3D
	LOGICAL		CIMNEY, OUTETA, INETA, PEMPHI, PEMDED
	LOGICAL		FEMETA, FEMPHI
	CHARACTER*3	REGION(3)
C--
C  Parameters defining fiducial regions and switches.
C--
        REAL            PHITOL, ZTKMIN
        LOGICAL         CUTTW9, CUTCEM, NEED3D, CUTCHI
        PARAMETER      (PHITOL = 21.0)    ! Maximum distance from tower center
        PARAMETER      (ZTKMIN = 9.)      ! Minimum strip chamber |Z|
        PARAMETER      (CUTTW9 = .TRUE.) ! Do we cut out Tower 9?
	PARAMETER      (CUTCHI = .TRUE.) ! Do we cut out Tower 7 in chimney?
        PARAMETER      (CUTCEM = .TRUE.) ! Do we require CEM seed tower?
        PARAMETER      (NEED3D = .TRUE.) ! Do we need 3D track?
C
C===External Declarations:
C
        REAL            CDFPHI
        INTEGER         BLOCAT
        INTEGER         IBELES, IDELES
        CHARACTER*8     ERRSTR
C
C===Data Statements:
C
        DATA REGION/'CEM','PEM','FEM'/
	DATA DEDARR/32,54,55,31,55,54,53,53,
     &              53,23,54,54,56,19,54,56,
     &		    69,24,24,15,0 ,31,30,31,
     &		    32,59,32,32,26,17,30,35/
C
C===ASF's
C
C       None
C
C===Executable Code:
C
        FIDELE = ERSUCC
        LFIDUC = .FALSE.
        FAILED = '1FFF'X
C--
C  Initialize logical cuts
C--
	INPHCR = .FALSE.
	IN90CR = .FALSE.
	TOWER9 = .FALSE.
	NOTCEM = .FALSE.
	NOT3D  = .FALSE.
	CIMNEY = .FALSE.
	OUTETA = .FALSE.
	INETA  = .FALSE.
	PEMPHI = .FALSE.
	PEMDED = .FALSE.
	FEMETA = .FALSE.
	FEMPHI = .FALSE.
C--
C  Find the corresponding ELES bank first.
C--
        STATUS = BLOCAT(IW, 'ELES', ELESBN, IBELES, IDELES)
        IF (STATUS.NE.YESUCC) THEN
          WRITE(ERRSTR(1:4),'(I4)') ELESBN
          CALL ERLOGR('FIDELE', ERWARN, 0, 0,
     >        'Could not locate ELES bank '//ERRSTR(1:4))
          FIDELE = ERWARN
          RETURN
        ENDIF
	IREG = IW(IDELES + REGELE)		! region (CEM 0, PEM 1, FEM 2)
C--
C  Get seed tower eta and phi
C--
        SEDETA = IAND(ISHFT(IW(IDELES + STWELE), -7), '7F'X)
	SEDPHI = IAND(IW(IDELES + STWELE), '7F'X)
C--
C  CEM clusters
C--
	IF (IREG.EQ.0) THEN
C--
C	Check if there are 3D tracks associated with this cluster
C--
          IF (IW(IDELES + N3DELE).GT.0) THEN
            NOT3D = .FALSE.
	  ELSE 
	    NOT3D = .TRUE.
	  ENDIF
C
C  Use strip locations to get local X and Z
C
C--- Calculate the local position of the cluster in X on CES and make
C    a fiducial cut.
C
          IPHI   = RW(IDELES+PSCELE) / (15.*DEGRAD)	! 0,1,..,23
          TEMP   = RW(IDELES+PSCELE) 
     &				- (FLOAT(IPHI) * 15. + 7.5) * DEGRAD
C
          WEDGEX = ABS(TAN(TEMP) * TRPOEC)
C--
C  Cut if local wedge x is outside phicut.
C--
          INPHCR = (WEDGEX.GT.PHITOL)
C--
C  Check if near the 90 degree crack (use CES position)
C--
          IN90CR = (ABS(RW(IDELES + ZSTELE)).LT.ZTKMIN)
C--
C  Check if seed tower is tower 9 or tower 7 of the chimney module
C--
          TOWER9 = (SEDETA.EQ.33 .OR. SEDETA.EQ.52)
	  CIMNEY = (SEDETA.EQ.50 .AND. SEDPHI.EQ.5)
C--
C  PEM clusters 
C	OUTETA = true if seed tower in outer 2 eta annuli
C	INETA  = true if seed tower in inner 2 eta annuli
C	PEMPHI = true if seed tower in border of quadrant
C	PEMDED = true if seed tower in dead channel 
C--	
	ELSEIF (IREG.EQ.1) THEN
	  OUTETA = (TETTYP(SEDETA).GE.3 .AND. TETTYP(SEDETA).LE.4)
	  INETA = (TETTYP(SEDETA).GE.7 .AND. TETTYP(SEDETA).LE.8)
	  PEMPHI = (SEDPHI.EQ.0 .OR. SEDPHI.EQ.17 .OR.
     >		    SEDPHI.EQ.18 .OR. SEDPHI.EQ.35 .OR.
     >		    SEDPHI.EQ.36 .OR. SEDPHI.EQ.53 .OR.
     >		    SEDPHI.EQ.54 .OR. SEDPHI.EQ.71)
C--
C  Check seed tower eta and phi against eta-phi array of dead
C  channels
C--
          DO 100 I = 1,DEDNUM
	    IF(SEDETA.LE.(DEDARR(I,1)+1) .AND.
     >		SEDETA.GE.(DEDARR(I,1)-1) .AND.
     >		SEDPHI.LE.(DEDARR(I,2)+1) .AND.
     >		SEDPHI.GE.(DEDARR(I,2)-1) ) PEMDED = .TRUE.
 100	  CONTINUE
C--
C  FEM clusters 
C	FEMETA  = true if seed tower in inner 5 eta annuli
C	FEMPHI = true if seed tower in border of quadrant
C--	
	ELSEIF (IREG.EQ.2) THEN
	  FEMETA = (SEDETA.LT.6 .OR. SEDETA.GT.79)
	  FEMPHI = (SEDPHI.EQ.0 .OR. SEDPHI.EQ.17 .OR.
     >		    SEDPHI.EQ.18 .OR. SEDPHI.EQ.35 .OR.
     >		    SEDPHI.EQ.36 .OR. SEDPHI.EQ.53 .OR.
     >		    SEDPHI.EQ.54 .OR. SEDPHI.EQ.71)
C--
C  Check for illegal region (< 0 , > 2)
C--
        ELSE
          FIDELE = ERWARN
	  RETURN
	END IF
C
C--
C  Now set the output flags.
C--
        FAILED = 0
        IF (INPHCR)		   FAILED = IBSET(FAILED,0)
        IF (IN90CR)		   FAILED = IBSET(FAILED,1)
        IF (CUTTW9 .AND. TOWER9)   FAILED = IBSET(FAILED,2)
        IF (CUTCHI .AND. CIMNEY)   FAILED = IBSET(FAILED,4)
        IF (OUTETA)		   FAILED = IBSET(FAILED,5)
        IF (INETA)		   FAILED = IBSET(FAILED,6)
        IF (PEMPHI)		   FAILED = IBSET(FAILED,7)
        IF (PEMDED)		   FAILED = IBSET(FAILED,8)
        IF (FEMETA)		   FAILED = IBSET(FAILED,9)
        IF (FEMPHI)		   FAILED = IBSET(FAILED,10)
        LFIDUC = (FAILED.EQ.0)
        IF (NEED3D .AND. NOT3D )   FAILED = IBSET(FAILED,3)
C--
C  Write out information if LUNIT > 0
C--
        IF (LUNIT.GT.0) THEN
	  WRITE(LUNIT,110) ELESBN, IREG
 110	  FORMAT(5X,'FIDELE: ELES Bank #:',I4,5x,'Region:',I4/)
	  IF(IREG.EQ.0) THEN
	      WRITE(LUNIT,120) WEDGEX,ABS(RW(IDELES+ZEXELE)),SEDETA,
     >			       SEDPHI,FAILED,LFIDUC
 120	      FORMAT(13X,'WEDGE X  ',F6.1,' (cm)'/,
     >		     13X,'WEDGE Z  ',F6.1,' (cm)'/,
     >		     13X,'Seed ETA ',I6/,
     >		     13X,'Seed PHI ',I6/,
     >		     13X,'FAILED   ',Z6,' (Hex)'/,
     >		     13X,'LFIDUC   ',L6/)
	  ELSEIF(IREG.GE.1) THEN
	      WRITE(LUNIT,130) SEDETA,SEDPHI,FAILED,LFIDUC
 130	      FORMAT(13X,'Seed ETA ',I6/,
     >		     13X,'Seed PHI ',I6/,
     >		     13X,'FAILED   ',Z6,' (Hex)'/,
     >		     13X,'LFIDUC   ',L6/)
	  ENDIF
        ENDIF
C==
        RETURN
        END
